package edu.lawrence.hotelfx;

import java.net.URL;
import java.time.LocalDate;
import java.util.ResourceBundle;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.DatePicker;
import javafx.scene.control.ListView;
import java.io.File;
import java.util.Scanner;
import edu.lawrence.hotel.Hotel;
import edu.lawrence.hotel.Room;
import edu.lawrence.hotel.Reservation;
import java.io.PrintWriter;
import javafx.application.Platform;
import javafx.collections.transformation.FilteredList;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.stage.Modality;
import javafx.stage.Stage;
import javafx.stage.StageStyle;

/**
 *
 * @author Joe Gregg
 */
public class FXMLDocumentController implements Initializable {
    private Hotel model;
    private Room currentRoom;
    private LocalDate selectedDate;
    
    @FXML
    private ListView reservationsList;
    
    @FXML
    private DatePicker datePicker;
    
    @FXML
    private ChoiceBox roomChoice;
    
    @FXML
    private void setDate(ActionEvent event) {
        selectedDate = datePicker.getValue();
        FilteredList<Reservation> filteredEvents = new FilteredList<Reservation>(currentRoom.getReservations(),(rsv)->rsv.getDate().equals(selectedDate));
        reservationsList.setItems(filteredEvents);
    }
    
    @FXML
    private void addReservation(ActionEvent event) {
        Stage parent = (Stage) reservationsList.getScene().getWindow();

        try {
            FXMLLoader loader = new FXMLLoader(getClass().getResource("FXMLNewEventDialog.fxml"));
            Parent root = (Parent)loader.load();
            Scene scene = new Scene(root);
            Stage dialog = new Stage();
            dialog.setScene(scene);
            dialog.setTitle("Create New Reservation");
            dialog.initOwner(parent);
            dialog.initModality(Modality.WINDOW_MODAL);
            dialog.initStyle(StageStyle.UTILITY);
            dialog.setX(parent.getX() + parent.getWidth()/4);
            dialog.setY(parent.getY() + parent.getHeight()/3);
            dialog.setWidth(450.0);
            dialog.setHeight(220.0);
            
            FXMLNewReservationDialogController controller = (FXMLNewReservationDialogController) loader.getController();
            controller.setRoom(currentRoom);
            controller.setDate(selectedDate);
            dialog.show();
        } catch(Exception ex) {
            System.out.println("Could not open dialog.");
            ex.printStackTrace();
        }
    }
    
    @FXML
    private void removeReservation(ActionEvent event) {
        Reservation toRemove = (Reservation) reservationsList.getSelectionModel().getSelectedItem();
        currentRoom.removeReservation(toRemove);
    }
    
    @FXML
    private void saveAll(ActionEvent event) {
        PrintWriter output = null;
        try { 
            output = new PrintWriter(new File("reservations.txt"));
            model.writeTo(output);
            output.close();
        } catch(Exception ex) {
            System.out.println("Error writing data to text file");
        }
    }
    
    @FXML
    private void exit(ActionEvent event) {
        Platform.exit();
    }
    
    private void setRoom(Object newRoom) {
        currentRoom = model.getRoom(newRoom.toString());
        FilteredList<Reservation> filteredEvents = new FilteredList<Reservation>(currentRoom.getReservations(),(rsv)->rsv.getDate().equals(selectedDate));
        reservationsList.setItems(filteredEvents);
    }
    
    @Override
    public void initialize(URL url, ResourceBundle rb) {
        selectedDate = LocalDate.now();
        datePicker.setValue(selectedDate);
        roomChoice.getItems().addAll("Grand Ballroom","Oneida","Michigan","Mendota","Sunrise");
        roomChoice.setValue("Grand Ballroom");
        roomChoice.getSelectionModel().selectedItemProperty().addListener((observable, oldValue, newValue)->setRoom(newValue));
        
        model = new Hotel();
        Scanner input = null;
        try {
            input = new Scanner(new File("reservations.txt"));
            model.readFrom(input);
            input.close();
            currentRoom = model.getRoom("Grand Ballroom");
        } catch (Exception ex) {
            System.out.println("Can not load data file.");
            ex.printStackTrace();
            if(input != null)
                input.close();
        }
        
        FilteredList<Reservation> filteredReservations = new FilteredList<Reservation>(currentRoom.getReservations(),
                (event)->event.getDate().equals(selectedDate));
        reservationsList.setItems(filteredReservations);
    }    
    
}
